import google.generativeai as genai
from core.errors import JSONParsingError, ConfigurationError
from core.helpers import remove_trailing_commas
from google.generativeai.types import GenerationConfig
import re
import json
from providers import AIProvider

class GeminiAIProvider(AIProvider):
    def __init__(self, config, provider_name):
        super().__init__(config, provider_name)
        self.load_provider_config()  # Load the config immediately after calling super

        # Access attributes directly
        genai.configure(api_key=self.provider_config.api_key)
        self.model = genai.GenerativeModel(self.provider_config.model)
        self.generation_config = GenerationConfig(
            temperature=float(self.provider_config.parameters.get("temperature", 0.1)), # Still use .get() for optional parameters
        )

    def load_provider_config(self):
        """Loads the Gemini-specific configuration."""
        self.provider_config = self.config.get_provider_config("gemini")
        if not self.provider_config.api_key:  # Access api_key directly
            pass# raise ConfigurationError("Gemini API key is not configured. Please set the api_key under [providers.gemini] in config.ini.")

    def generate_content(self, prompt: str) -> str:
        try:
            response = self.model.generate_content(prompt, generation_config=self.generation_config)
            return response.text
        except Exception as e:
            raise Exception(f"Error generating content with Gemini: {e}")

    def parse_response(self, response: str) -> dict:
        try:
            match = re.search(r"\{.*\}", response, re.DOTALL)
            if not match:
                raise ValueError("No JSON object found in response.")
            json_string = match.group(0)
            cleaned_response = remove_trailing_commas(json_string)
            return json.loads(cleaned_response)
        except json.JSONDecodeError as e:
            raise JSONParsingError(f"Error decoding JSON: {e}", response)